@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -
 #
-# This file is part of gunicorn released under the MIT license. 
+# This file is part of gunicorn released under the MIT license.
 # See the NOTICE for more information.
 #
 # Example code from Eventlet sources
@@ -8,7 +8,8 @@
 import os
 from wsgiref.validate import validator
 
-#@validator
+
+@validator
 def app(environ, start_response):
     """Simplest possible application object"""
     status = '200 OK'
@@ -16,7 +17,7 @@ def app(environ, start_response):
     f = open(fname, 'rb')
 
     response_headers = [
-        ('Content-type','text/plain'),
+        ('Content-type', 'text/plain'),
     ]
     start_response(status, response_headers)
 
