@@ -9,9 +9,9 @@ import os
 import re
 import sys
 
-from gunicorn.six import unquote_to_wsgi_str, string_types, binary_type, reraise
+from gunicorn._compat import unquote_to_wsgi_str
+from gunicorn.six import string_types, binary_type, reraise
 from gunicorn import SERVER_SOFTWARE
-import gunicorn.six as six
 import gunicorn.util as util
 
 try:
@@ -23,6 +23,10 @@ except ImportError:
     except ImportError:
         sendfile = None
 
+# Send files in at most 1GB blocks as some operating systems can have problems
+# with sending files in blocks over 2GB.
+BLKSIZE = 0x3FFFFFFF
+
 NORMALIZE_SPACE = re.compile(r'(?:\r\n)?[ \t]+')
 
 log = logging.getLogger(__name__)
@@ -43,7 +47,7 @@ class FileWrapper(object):
         raise IndexError
 
 
-class WSGIErrorsWraper(io.RawIOBase):
+class WSGIErrorsWrapper(io.RawIOBase):
 
     def __init__(self, cfg):
         errorlog = logging.getLogger("gunicorn.error")
@@ -69,7 +73,7 @@ class WSGIErrorsWraper(io.RawIOBase):
 
 def base_environ(cfg):
     return {
-        "wsgi.errors": WSGIErrorsWraper(cfg),
+        "wsgi.errors": WSGIErrorsWrapper(cfg),
         "wsgi.version": (1, 0),
         "wsgi.multithread": False,
         "wsgi.multiprocess": (cfg.workers > 1),
@@ -101,7 +105,7 @@ def proxy_environ(req):
     return {
         "PROXY_PROTOCOL": info["proxy_protocol"],
         "REMOTE_ADDR": info["client_addr"],
-        "REMOTE_PORT":  str(info["client_port"]),
+        "REMOTE_PORT": str(info["client_port"]),
         "PROXY_ADDR": info["proxy_addr"],
         "PROXY_PORT": str(info["proxy_port"]),
     }
@@ -125,7 +129,7 @@ def create(req, sock, client, server, cfg):
                 and client[0] not in cfg.forwarded_allow_ips):
             secure_headers = {}
 
-    # add the headers tot the environ
+    # add the headers to the environ
     for hdr_name, hdr_value in req.headers:
         if hdr_name == "EXPECT":
             # handle expect
@@ -150,7 +154,7 @@ def create(req, sock, client, server, cfg):
             hdr_value = "%s,%s" % (environ[key], hdr_value)
         environ[key] = hdr_value
 
-    # set the url schejeme
+    # set the url scheme
     environ['wsgi.url_scheme'] = url_scheme
 
     # set the REMOTE_* keys in environ
@@ -256,8 +260,8 @@ class Response(object):
 
     def process_headers(self, headers):
         for name, value in headers:
-            assert isinstance(name, string_types), "%r is not a string" % name
-
+            if not isinstance(name, string_types):
+                raise TypeError('%r is not a string' % name)
             value = str(value).strip()
             lname = name.lower().strip()
             if lname == "content-length":
@@ -316,14 +320,13 @@ class Response(object):
         tosend.extend(["%s: %s\r\n" % (k, v) for k, v in self.headers])
 
         header_str = "%s\r\n" % "".join(tosend)
-        util.write(self.sock, util.to_bytestring(header_str))
+        util.write(self.sock, util.to_bytestring(header_str, "ascii"))
         self.headers_sent = True
 
     def write(self, arg):
         self.send_headers()
-
-        assert isinstance(arg, binary_type), "%r is not a byte." % arg
-
+        if not isinstance(arg, binary_type):
+            raise TypeError('%r is not a byte' % arg)
         arglen = len(arg)
         tosend = arglen
         if self.response_length is not None:
@@ -343,75 +346,55 @@ class Response(object):
         self.sent += tosend
         util.write(self.sock, arg, self.chunked)
 
-    def sendfile_all(self, fileno, sockno, offset, nbytes):
-        # Send file in at most 1GB blocks as some operating
-        # systems can have problems with sending files in blocks
-        # over 2GB.
+    def can_sendfile(self):
+        return self.cfg.sendfile is not False and sendfile is not None
 
-        BLKSIZE = 0x3FFFFFFF
+    def sendfile(self, respiter):
+        if self.cfg.is_ssl or not self.can_sendfile():
+            return False
 
-        if nbytes > BLKSIZE:
-            for m in range(0, nbytes, BLKSIZE):
-                self.sendfile_all(fileno, sockno, offset, min(nbytes, BLKSIZE))
-                offset += BLKSIZE
-                nbytes -= BLKSIZE
-        else:
-            sent = 0
-            sent += sendfile(sockno, fileno, offset + sent, nbytes - sent)
-            while sent != nbytes:
-                sent += sendfile(sockno, fileno, offset + sent, nbytes - sent)
+        if not util.has_fileno(respiter.filelike):
+            return False
 
-    def sendfile_use_send(self, fileno, fo_offset, nbytes):
+        fileno = respiter.filelike.fileno()
+        try:
+            offset = os.lseek(fileno, 0, os.SEEK_CUR)
+            if self.response_length is None:
+                filesize = os.fstat(fileno).st_size
 
-        # send file in blocks of 8182 bytes
-        BLKSIZE = 8192
+                # The file may be special and sendfile will fail.
+                # It may also be zero-length, but that is okay.
+                if filesize == 0:
+                    return False
 
+                nbytes = filesize - offset
+            else:
+                nbytes = self.response_length
+        except (OSError, io.UnsupportedOperation):
+            return False
+
+        self.send_headers()
+
+        if self.is_chunked():
+            chunk_size = "%X\r\n" % nbytes
+            self.sock.sendall(chunk_size.encode('utf-8'))
+
+        sockno = self.sock.fileno()
         sent = 0
+
         while sent != nbytes:
-            data = os.read(fileno, BLKSIZE)
-            if not data:
-                break
+            count = min(nbytes - sent, BLKSIZE)
+            sent += sendfile(sockno, fileno, offset + sent, count)
 
-            sent += len(data)
-            if sent > nbytes:
-                data = data[:nbytes-sent]
+        if self.is_chunked():
+            self.sock.sendall(b"\r\n")
 
-            util.write(self.sock, data, self.chunked)
+        os.lseek(fileno, offset, os.SEEK_SET)
+
+        return True
 
     def write_file(self, respiter):
-        if sendfile is not None and util.is_fileobject(respiter.filelike):
-            # sometimes the fileno isn't a callable
-            if six.callable(respiter.filelike.fileno):
-                fileno = respiter.filelike.fileno()
-            else:
-                fileno = respiter.filelike.fileno
-
-            fd_offset = os.lseek(fileno, 0, os.SEEK_CUR)
-            fo_offset = respiter.filelike.tell()
-            nbytes = max(os.fstat(fileno).st_size - fo_offset, 0)
-
-            if self.response_length:
-                nbytes = min(nbytes, self.response_length)
-
-            if nbytes == 0:
-                return
-
-            self.send_headers()
-
-            if self.cfg.is_ssl:
-                self.sendfile_use_send(fileno, fo_offset, nbytes)
-            else:
-                if self.is_chunked():
-                    chunk_size = "%X\r\n" % nbytes
-                    self.sock.sendall(chunk_size.encode('utf-8'))
-
-                self.sendfile_all(fileno, self.sock.fileno(), fo_offset, nbytes)
-
-                if self.is_chunked():
-                    self.sock.sendall(b"\r\n")
-
-            os.lseek(fileno, fd_offset, os.SEEK_SET)
-        else:
+        if not self.sendfile(respiter):
             for item in respiter:
                 self.write(item)
 
