@@ -7,7 +7,10 @@
 
 import socket
 import logging
+from re import sub
+
 from gunicorn.glogging import Logger
+from gunicorn import six
 
 # Instrumentation constants
 STATSD_DEFAULT_PORT = 8125
@@ -25,6 +28,7 @@ class Statsd(Logger):
         """host, port: statsD server
         """
         Logger.__init__(self, cfg)
+        self.prefix = sub(r"^(.+[^.]+)\.*$", "\g<1>.", cfg.statsd_prefix)
         try:
             host, port = cfg.statsd_host
             self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
@@ -75,13 +79,12 @@ class Statsd(Logger):
                         self.histogram(metric, value)
                     else:
                         pass
-             
+
             # Log to parent logger only if there is something to say
             if msg is not None and len(msg) > 0:
                 Logger.log(self, lvl, msg, *args, **kwargs)
         except Exception:
-            pass
-
+            Logger.warning(self, "Failed to log to statsd", exc_info=True)
 
     # access logging
     def access(self, resp, req, environ, request_time):
@@ -89,7 +92,7 @@ class Statsd(Logger):
         request_time is a datetime.timedelta
         """
         Logger.access(self, resp, req, environ, request_time)
-        duration_in_ms = request_time.seconds * 1000 + float(request_time.microseconds)/10**3
+        duration_in_ms = request_time.seconds * 1000 + float(request_time.microseconds) / 10 ** 3
         self.histogram("gunicorn.request.duration", duration_in_ms)
         self.increment("gunicorn.requests", 1)
         self.increment("gunicorn.request.status.%d" % int(resp.status.split()[0]), 1)
@@ -97,29 +100,22 @@ class Statsd(Logger):
     # statsD methods
     # you can use those directly if you want
     def gauge(self, name, value):
-        try:
-            if self.sock:
-                self.sock.send("{0}:{1}|g".format(name, value))
-        except Exception:
-            pass
+        self._sock_send("{0}{1}:{2}|g".format(self.prefix, name, value))
 
     def increment(self, name, value, sampling_rate=1.0):
-        try:
-            if self.sock:
-                self.sock.send("{0}:{1}|c|@{2}".format(name, value, sampling_rate))
-        except Exception:
-            pass
+        self._sock_send("{0}{1}:{2}|c|@{3}".format(self.prefix, name, value, sampling_rate))
 
     def decrement(self, name, value, sampling_rate=1.0):
-        try:
-            if self.sock:
-                self.sock.send("{0}:-{1}|c|@{2}".format(name, value, sampling_rate))
-        except Exception:
-            pass
+        self._sock_send("{0){1}:-{2}|c|@{3}".format(self.prefix, name, value, sampling_rate))
 
     def histogram(self, name, value):
+        self._sock_send("{0}{1}:{2}|ms".format(self.prefix, name, value))
+
+    def _sock_send(self, msg):
         try:
+            if isinstance(msg, six.text_type):
+                msg = msg.encode("ascii")
             if self.sock:
-                self.sock.send("{0}:{1}|ms".format(name, value))
+                self.sock.send(msg)
         except Exception:
-            pass
+            Logger.warning(self, "Error sending message to statsd", exc_info=True)
