@@ -3,15 +3,19 @@
 # This file is part of gunicorn released under the MIT license.
 # See the NOTICE for more information.
 
+from __future__ import print_function
 
 import email.utils
 import fcntl
+import grp
 import io
 import os
 import pkg_resources
+import pwd
 import random
 import resource
 import socket
+import stat
 import sys
 import textwrap
 import time
@@ -25,7 +29,6 @@ from gunicorn.errors import AppImportError
 from gunicorn.six import text_type
 from gunicorn.workers import SUPPORTED_WORKERS
 
-
 MAXFD = 1024
 REDIRECT_TO = getattr(os, 'devnull', '/dev/null')
 
@@ -70,8 +73,8 @@ except ImportError:
             try:
                 dot = package.rindex('.', 0, dot)
             except ValueError:
-                raise ValueError("attempted relative import beyond top-level "
-                                  "package")
+                msg = "attempted relative import beyond top-level package"
+                raise ValueError(msg)
         return "%s.%s" % (package[:dot], name)
 
     def import_module(name, package=None):
@@ -112,8 +115,8 @@ def load_class(uri, default="gunicorn.workers.sync.SyncWorker",
             return pkg_resources.load_entry_point(dist, section, name)
         except:
             exc = traceback.format_exc()
-            raise RuntimeError("class uri %r invalid or not found: \n\n[%s]" % (uri,
-                exc))
+            msg = "class uri %r invalid or not found: \n\n[%s]"
+            raise RuntimeError(msg % (uri, exc))
     else:
         components = uri.split('.')
         if len(components) == 1:
@@ -130,8 +133,8 @@ def load_class(uri, default="gunicorn.workers.sync.SyncWorker",
                                 section, uri)
                 except:
                     exc = traceback.format_exc()
-                    raise RuntimeError("class uri %r invalid or not found: \n\n[%s]" % (uri,
-                        exc))
+                    msg = "class uri %r invalid or not found: \n\n[%s]"
+                    raise RuntimeError(msg % (uri, exc))
 
         klass = components.pop(-1)
 
@@ -139,9 +142,8 @@ def load_class(uri, default="gunicorn.workers.sync.SyncWorker",
             mod = import_module('.'.join(components))
         except:
             exc = traceback.format_exc()
-            raise RuntimeError(
-                    "class uri %r invalid or not found: \n\n[%s]" %
-                    (uri, exc))
+            msg = "class uri %r invalid or not found: \n\n[%s]"
+            raise RuntimeError(msg % (uri, exc))
         return getattr(mod, klass)
 
 
@@ -215,7 +217,7 @@ def is_ipv6(addr):
         socket.inet_pton(socket.AF_INET6, addr)
     except socket.error:  # not a valid address
         return False
-    except ValueError: # ipv6 not supported on this platform
+    except ValueError:  # ipv6 not supported on this platform
         return False
     return True
 
@@ -230,7 +232,6 @@ def parse_address(netloc, default_port=8000):
     if netloc.startswith("tcp://"):
         netloc = netloc.split("tcp://")[1]
 
-
     # get host
     if '[' in netloc and ']' in netloc:
         host = netloc.split(']')[0][1:].lower()
@@ -356,8 +357,8 @@ def import_app(module):
         __import__(module)
     except ImportError:
         if module.endswith(".py") and os.path.exists(module):
-            raise ImportError("Failed to find application, did "
-                "you mean '%s:%s'?" % (module.rsplit(".", 1)[0], obj))
+            msg = "Failed to find application, did you mean '%s:%s'?"
+            raise ImportError(msg % (module.rsplit(".", 1)[0], obj))
         else:
             raise
 
@@ -407,7 +408,7 @@ def daemonize(enable_stdio_inheritance=False):
     Standard daemonization of a process.
     http://www.svbug.com/documentation/comp.unix.programmer-FAQ/faq_2.html#SEC16
     """
-    if not 'GUNICORN_FD' in os.environ:
+    if 'GUNICORN_FD' not in os.environ:
         if os.fork():
             os._exit(0)
         os.setsid()
@@ -415,7 +416,7 @@ def daemonize(enable_stdio_inheritance=False):
         if os.fork():
             os._exit(0)
 
-        os.umask(0)
+        os.umask(0o22)
 
         # In both the following any file descriptors above stdin
         # stdout and stderr are left untouched. The inheritence
@@ -500,35 +501,48 @@ def check_is_writeable(path):
     f.close()
 
 
-def to_bytestring(value):
+def to_bytestring(value, encoding="utf8"):
     """Converts a string argument to a byte string"""
     if isinstance(value, bytes):
         return value
-    assert isinstance(value, text_type)
-    return value.encode("utf-8")
+    if not isinstance(value, text_type):
+        raise TypeError('%r is not a string' % value)
 
+    return value.encode(encoding)
 
-def is_fileobject(obj):
-    if not hasattr(obj, "tell") or not hasattr(obj, "fileno"):
+def has_fileno(obj):
+    if not hasattr(obj, "fileno"):
         return False
 
     # check BytesIO case and maybe others
     try:
         obj.fileno()
-    except (IOError, io.UnsupportedOperation):
+    except (AttributeError, IOError, io.UnsupportedOperation):
         return False
 
     return True
 
 
 def warn(msg):
-    sys.stderr.write("!!!\n")
+    print("!!!", file=sys.stderr)
 
     lines = msg.splitlines()
     for i, line in enumerate(lines):
         if i == 0:
             line = "WARNING: %s" % line
-        sys.stderr.write("!!! %s\n" % line)
+        print("!!! %s" % line, file=sys.stderr)
 
-    sys.stderr.write("!!!\n\n")
+    print("!!!\n", file=sys.stderr)
     sys.stderr.flush()
+
+
+def make_fail_app(msg):
+
+    def app(environ, start_response):
+        start_response("500 Internal Server Error", [
+            ("Content-Type", "text/plain"),
+            ("Content-Length", str(len(msg)))
+        ])
+        return [msg]
+
+    return app
