@@ -7,14 +7,18 @@ from datetime import datetime
 import os
 import signal
 import sys
+import time
+import traceback
+from random import randint
 
 
 from gunicorn import util
 from gunicorn.workers.workertmp import WorkerTmp
 from gunicorn.reloader import Reloader
-from gunicorn.http.errors import InvalidHeader, InvalidHeaderName, \
-InvalidRequestLine, InvalidRequestMethod, InvalidHTTPVersion, \
-LimitRequestLine, LimitRequestHeaders
+from gunicorn.http.errors import (
+    InvalidHeader, InvalidHeaderName, InvalidRequestLine, InvalidRequestMethod,
+    InvalidHTTPVersion, LimitRequestLine, LimitRequestHeaders,
+)
 from gunicorn.http.errors import InvalidProxyLine, ForbiddenProxyRequest
 from gunicorn.http.wsgi import default_environ, Response
 from gunicorn.six import MAXSIZE
@@ -22,7 +26,7 @@ from gunicorn.six import MAXSIZE
 
 class Worker(object):
 
-    SIGNALS = [getattr(signal, "SIG%s" % x) \
+    SIGNALS = [getattr(signal, "SIG%s" % x)
             for x in "ABRT HUP QUIT INT TERM USR1 USR2 WINCH CHLD".split()]
 
     PIPE = []
@@ -41,9 +45,11 @@ class Worker(object):
         self.cfg = cfg
         self.booted = False
         self.aborted = False
+        self.reloader = None
 
         self.nr = 0
-        self.max_requests = cfg.max_requests or MAXSIZE
+        jitter = randint(0, cfg.max_requests_jitter)
+        self.max_requests = cfg.max_requests + jitter or MAXSIZE
         self.alive = True
         self.log = log
         self.tmp = WorkerTmp(cfg)
@@ -84,8 +90,8 @@ class Worker(object):
             def changed(fname):
                 self.log.info("Worker reloading: %s modified", fname)
                 os.kill(self.pid, signal.SIGQUIT)
-                raise SystemExit()
-            Reloader(callback=changed).start()
+            self.reloader = Reloader(callback=changed)
+            self.reloader.start()
 
         # set environment' variables
         if self.cfg.env:
@@ -111,14 +117,29 @@ class Worker(object):
 
         self.init_signals()
 
-        self.wsgi = self.app.wsgi()
-
         self.cfg.post_worker_init(self)
 
+        self.load_wsgi()
+
         # Enter main run loop
         self.booted = True
         self.run()
 
+    def load_wsgi(self):
+        try:
+            self.wsgi = self.app.wsgi()
+        except SyntaxError as e:
+            if not self.cfg.reload:
+                raise
+
+            self.log.exception(e)
+
+            exc_type, exc_val, exc_tb = sys.exc_info()
+            self.reloader.add_extra_file(exc_val.filename)
+
+            tb_string = traceback.format_exc(exc_tb)
+            self.wsgi = util.make_fail_app(tb_string)
+
     def init_signals(self):
         # reset signaling
         [signal.signal(s, signal.SIG_DFL) for s in self.SIGNALS]
@@ -146,6 +167,7 @@ class Worker(object):
         self.alive = False
         # worker_int callback
         self.cfg.worker_int(self)
+        time.sleep(0.1)
         sys.exit(0)
 
     def handle_abort(self, sig, frame):
@@ -157,9 +179,9 @@ class Worker(object):
         request_start = datetime.now()
         addr = addr or ('', -1)  # unix socket case
         if isinstance(exc, (InvalidRequestLine, InvalidRequestMethod,
-            InvalidHTTPVersion, InvalidHeader, InvalidHeaderName,
-            LimitRequestLine, LimitRequestHeaders,
-            InvalidProxyLine, ForbiddenProxyRequest,)):
+                InvalidHTTPVersion, InvalidHeader, InvalidHeaderName,
+                LimitRequestLine, LimitRequestHeaders,
+                InvalidProxyLine, ForbiddenProxyRequest)):
 
             status_int = 400
             reason = "Bad Request"
@@ -185,13 +207,10 @@ class Worker(object):
                 mesg = "Request forbidden"
                 status_int = 403
 
-            self.log.debug("Invalid request from ip={ip}: {error}"\
-                           "".format(ip=addr[0],
-                                     error=str(exc),
-                                    )
-                          )
+            msg = "Invalid request from ip={ip}: {error}"
+            self.log.debug(msg.format(ip=addr[0], error=str(exc)))
         else:
-            self.log.exception("Error handling request")
+            self.log.exception("Error handling request %s", req.uri)
 
             status_int = 500
             reason = "Internal Server Error"
