@@ -7,5 +7,5 @@ request = {
         ("CONTENT-TYPE", "application/json"),
         ("CONTENT-LENGTH", "14")
     ],
-    "body": '{"nom": "nom"}'
-}
\ No newline at end of file
+    "body": b'{"nom": "nom"}'
+}
