@@ -4,7 +4,6 @@
 # This file is part of gunicorn released under the MIT license.
 # See the NOTICE for more information.
 
-import array
 import os
 import tempfile
 
@@ -78,40 +77,3 @@ class http_request(object):
             func(req)
         run.func_name = func.func_name
         return run
-
-def eq(a, b):
-    assert a == b, "%r != %r" % (a, b)
-
-def ne(a, b):
-    assert a != b, "%r == %r" % (a, b)
-
-def lt(a, b):
-    assert a < b, "%r >= %r" % (a, b)
-
-def gt(a, b):
-    assert a > b, "%r <= %r" % (a, b)
-
-def isin(a, b):
-    assert a in b, "%r is not in %r" % (a, b)
-
-def isnotin(a, b):
-    assert a not in b, "%r is in %r" % (a, b)
-
-def has(a, b):
-    assert hasattr(a, b), "%r has no attribute %r" % (a, b)
-
-def hasnot(a, b):
-    assert not hasattr(a, b), "%r has an attribute %r" % (a, b)
-
-def istype(a, b):
-    assert isinstance(a, b), "%r is not an instance of %r" % (a, b)
-
-def raises(exctype, func, *args, **kwargs):
-    try:
-        func(*args, **kwargs)
-    except exctype:
-        pass
-    else:
-        func_name = getattr(func, "func_name", "<builtin_function>")
-        raise AssertionError("Function %s did not raise %s" % (
-            func_name, exctype.__name__))
